use std::path::Path;

use based::asset::AssetRoutes;
use based::page;
use based::request::{RequestContext, StringResponse};
use based::ui::components::{Shell, Tabs};
use based::ui::prelude::*;
use based::ui::primitives::flex::Row;
use based::ui::primitives::Optional;
use config::{Project, ProjectConfig};
use maud::Render;
use rocket::fs::NamedFile;
use rocket::request::FromSegments;
use rocket::{get, launch, routes, State};
use site::gen_shell;

mod config;
mod invest;
mod site;

pub struct PathSegment {
    pub segments: Vec<String>,
}

impl PathSegment {
    pub fn to_str(&self) -> String {
        self.segments.join("/")
    }
}

impl<'r> FromSegments<'r> for PathSegment {
    type Error = ();

    fn from_segments(
        segments: rocket::http::uri::Segments<'r, rocket::http::uri::fmt::Path>,
    ) -> Result<Self, Self::Error> {
        let paths: Vec<_> = segments
            .filter_map(|x| {
                if x == "." {
                    return None;
                }

                if x == ".." {
                    return None;
                }

                Some(x.to_string())
            })
            .collect();

        Ok(PathSegment { segments: paths })
    }
}

#[get("/prj/<path..>")]
pub async fn project_page(
    path: PathSegment,
    ctx: RequestContext,
    shell: &State<Shell>,
    c: &State<ProjectConfig>,
) -> Option<StringResponse> {
    let mut prj = c.get(path.segments.first()?);
    for p in &path.segments[1..] {
        println!(" --> {p}");
        prj = prj.as_ref()?.sub.as_ref()?.get(p);
    }

    Some(render_project(shell, ctx, prj?, path.to_str()).await)
}

#[get("/static/<file>")]
async fn icon_res(file: &str) -> Option<NamedFile> {
    let path = Path::new("static").join(file);
    NamedFile::open(path).await.ok()
}

#[get("/")]
pub async fn main_page(
    c: &State<ProjectConfig>,
    ctx: RequestContext,
    shell: &State<Shell>,
) -> StringResponse {
    if c.len() == 1 {
        let keys = c.keys().collect::<Vec<_>>();
        let first = keys.first().unwrap();
        let prj = c.get(*first).unwrap();

        render_project(shell, ctx, prj, format!("{first}/")).await
    } else {
        // TODO : root project overview
        site::gen_site(c, ctx).await
    }
}

pub async fn render_project(
    shell: &State<Shell>,
    ctx: RequestContext,
    prj: &Project,
    root: String,
) -> StringResponse {
    let title = format!("{} - Umbrella ☂️", prj.name);

    page!(
        shell,
        ctx,
        title,
        Div()
            .vanish()
            .push(
                Margin(
                    Row(vec![
                        Optional(prj.icon.as_ref(), |icon| Image(icon).alt("Project Icon")),
                        Text(&prj.name)._2xl().render()
                    ])
                    .gap(ScreenValue::_4)
                    .full_center()
                )
                .top(ScreenValue::_6)
                .bottom(ScreenValue::_2)
            )
            .push(
                Padding(
                    Tabs()
                        .add_tab(
                            "info",
                            Text("📜 Info").medium(),
                            Flex(
                                Div()
                                    .vanish()
                                    .push(Text(&prj.description))
                                    .push(Optional(prj.website.as_deref(), |website| Link(
                                        website,
                                        Text(&format!("Website: {website}"))
                                    )))
                                    .push(Optional(prj.documentation.as_deref(), |docs| Link(
                                        docs,
                                        Text(&format!("Documentation: {docs}"))
                                    )))
                                    .push(Optional(prj.since.as_deref(), |since| Text(&format!(
                                        "Since: {since}"
                                    ))))
                                    .push(Optional(prj.contact.as_ref(), |contact| {
                                        Div().vanish().push(Text("Contact").large()).push(Text(
                                            &format!(
                                                "eMail: {}",
                                                contact
                                                    .email
                                                    .as_ref()
                                                    .map(|x| x.as_str())
                                                    .unwrap_or_default()
                                            ),
                                        ))
                                    }))
                                    .push(Optional(prj.sub.as_ref(), |sub| {
                                        let mut prj_links = Vec::new();

                                        for key in sub.keys() {
                                            prj_links.push(key);
                                        }

                                        Div()
                                            .vanish()
                                            .push(Text("Sub projects").large())
                                            .push_for_each(&prj_links, |link: &_| {
                                                Link(
                                                    &format!("/prj/{root}{link}"),
                                                    Text(
                                                        &prj.sub
                                                            .as_ref()
                                                            .unwrap()
                                                            .get(*link)
                                                            .unwrap()
                                                            .name,
                                                    ),
                                                )
                                            })
                                    }))
                            )
                            .gap(ScreenValue::_2)
                            .direction(Direction::Column)
                        )
                        .add_tab(
                            "invest",
                            Text("💵 Invest").medium(),
                            Div().vanish().push(Text("TODO"))
                        )
                )
                .x(ScreenValue::_6)
            )
    )
}

// todo : fav icon

#[launch]
async fn rocket() -> _ {
    let conf_path: String = std::env::args()
        .skip(1)
        .next()
        .unwrap_or("./config.toml".to_string());
    let conf = config::Config::load(&conf_path);

    let shell = gen_shell();

    rocket::build()
        .mount_assets()
        .mount("/", routes![main_page, icon_res, project_page])
        .manage(conf)
        .manage(shell)
}
